@@ -5,10 +5,11 @@ import os
 import md4c
 import polib
 
-from mdpo.command import search_html_command
+from mdpo.command import parse_mdpo_html_command
 from mdpo.io import filter_paths, to_glob_or_content
 from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
-from mdpo.po import build_po_escaped_string
+from mdpo.po import find_entry_in_entries, po_escaped_string
+from mdpo.polib import *  # noqa
 from mdpo.text import min_not_max_chars_in_a_row
 
 
@@ -18,28 +19,32 @@ class Md2Po:
         if is_glob:
             self.filepaths = filter_paths(
                 glob_or_content,
-                ignore_paths=kwargs.get('ignore', []))
+                ignore_paths=kwargs.get('ignore', []),
+            )
         else:
             self.content = glob_or_content
 
         self.pofile = None
         self.msgstr = kwargs.get('msgstr', '')
-        self.msgids = []
+        self.found_entries = []
         self._current_msgid = ''
         self._current_tcomment = None
         self._current_msgctxt = None
 
         self.wrapwidth = kwargs.get('wrapwidth', 78)
 
-        self.mark_not_found_as_absolete = kwargs.get(
-            'mark_not_found_as_absolete', True)
+        self.mark_not_found_as_obsolete = kwargs.get(
+            'mark_not_found_as_obsolete', True,
+        )
 
-        self.extensions = kwargs.get("extensions",
-                                     DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS)
+        self.extensions = kwargs.get(
+            'extensions',
+            DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
+        )
 
         self.plaintext = kwargs.get('plaintext', False)
 
-        self.include_codeblocks = kwargs.get("include_codeblocks", False)
+        self.include_codeblocks = kwargs.get('include_codeblocks', False)
 
         self._disable = False
         self._disable_next_line = False
@@ -49,47 +54,53 @@ class Md2Po:
 
         if not self.plaintext:
             self.bold_start_string = kwargs.get('bold_start_string', '**')
-            self.bold_start_string_escaped = build_po_escaped_string(
-                self.bold_start_string)
+            self.bold_start_string_escaped = po_escaped_string(
+                self.bold_start_string,
+            )
 
             self.bold_end_string = kwargs.get('bold_end_string', '**')
-            self.bold_end_string_escaped = build_po_escaped_string(
-                self.bold_end_string)
+            self.bold_end_string_escaped = po_escaped_string(
+                self.bold_end_string,
+            )
 
             self.italic_start_string = kwargs.get('italic_start_string', '*')
-            self.italic_start_string_escaped = build_po_escaped_string(
-                self.italic_start_string)
+            self.italic_start_string_escaped = po_escaped_string(
+                self.italic_start_string,
+            )
 
             self.italic_end_string = kwargs.get('italic_end_string', '*')
-            self.italic_end_string_escaped = build_po_escaped_string(
-                self.italic_end_string)
+            self.italic_end_string_escaped = po_escaped_string(
+                self.italic_end_string,
+            )
 
             self._bold_italic_context = False
 
             # codespans are built by a indetermined number of 'x' characters
             # so we take only the first
             self.code_start_string = kwargs.get('code_start_string', '`')[0]
-            self.code_start_string_escaped = build_po_escaped_string(
-                self.code_start_string)
+            self.code_start_string_escaped = po_escaped_string(
+                self.code_start_string,
+            )
 
             self.code_end_string = kwargs.get('code_end_string', '`')[0]
-            self.code_end_string_escaped = build_po_escaped_string(
-                self.code_end_string)
+            self.code_end_string_escaped = po_escaped_string(
+                self.code_end_string,
+            )
 
             self.link_start_string = kwargs.get('link_start_string', '[')
             self.link_end_string = kwargs.get('link_end_string', ']')
 
-            self._include_xheaders = kwargs.get("xheaders", False)
+            self._include_xheaders = kwargs.get('xheaders', False)
             if self._include_xheaders:
                 self.xheaders = {
-                    "x-mdpo-bold-start": self.bold_start_string,
-                    "x-mdpo-bold-end": self.bold_end_string,
-                    "x-mdpo-italic-start": self.italic_start_string,
-                    "x-mdpo-italic-end": self.italic_end_string,
-                    "x-mdpo-code-start": self.code_start_string,
-                    "x-mdpo-code-end": self.code_end_string,
-                    "x-mdpo-link-start": self.link_start_string,
-                    "x-mdpo-link-end": self.link_end_string,
+                    'x-mdpo-bold-start': self.bold_start_string,
+                    'x-mdpo-bold-end': self.bold_end_string,
+                    'x-mdpo-italic-start': self.italic_start_string,
+                    'x-mdpo-italic-end': self.italic_end_string,
+                    'x-mdpo-code-start': self.code_start_string,
+                    'x-mdpo-code-end': self.code_end_string,
+                    'x-mdpo-link-start': self.link_start_string,
+                    'x-mdpo-link-end': self.link_end_string,
                 }
 
             self._enterspan_replacer = {
@@ -106,61 +117,69 @@ class Md2Po:
                 md4c.SpanType.A: self.link_end_string,
             }
 
-            if "strikethrough" in self.extensions:
+            if 'strikethrough' in self.extensions:
                 self.strikethrough_start_string = kwargs.get(
-                    'strikethrough_start_string', '~~')
+                    'strikethrough_start_string', '~~',
+                )
                 self._enterspan_replacer[md4c.SpanType.DEL] = \
                     self.strikethrough_start_string
 
                 self.strikethrough_end_string = kwargs.get(
-                    'strikethrough_end_string', '~~')
+                    'strikethrough_end_string', '~~',
+                )
                 self._leavespan_replacer[md4c.SpanType.DEL] = \
                     self.strikethrough_end_string
 
                 if self._include_xheaders:
                     self.xheaders.update({
-                        "x-mdpo-strikethrough-start":
+                        'x-mdpo-strikethrough-start':
                             self.strikethrough_start_string,
-                        "x-mdpo-strikethrough-end":
+                        'x-mdpo-strikethrough-end':
                             self.strikethrough_end_string,
                     })
 
-            if "latex_math_spans" in self.extensions:
+            if 'latex_math_spans' in self.extensions:
                 self.latexmath_start_string = kwargs.get(
-                    'latexmath_start_string', '$')
+                    'latexmath_start_string', '$',
+                )
                 self._enterspan_replacer[md4c.SpanType.LATEXMATH] = \
                     self.latexmath_start_string
 
                 self.latexmath_end_string = kwargs.get(
-                    'latexmath_end_string', '$')
+                    'latexmath_end_string', '$',
+                )
                 self._leavespan_replacer[md4c.SpanType.LATEXMATH] = \
                     self.latexmath_end_string
 
                 self.latexmathdisplay_start_string = kwargs.get(
-                    'latexmathdisplay_start_string', '$$')
+                    'latexmathdisplay_start_string', '$$',
+                )
                 self._enterspan_replacer[md4c.SpanType.LATEXMATH_DISPLAY] = \
                     self.latexmathdisplay_start_string
 
                 self.latexmathdisplay_end_string = kwargs.get(
-                    'latexmathdisplay_end_string', '$$')
+                    'latexmathdisplay_end_string', '$$',
+                )
                 self._leavespan_replacer[md4c.SpanType.LATEXMATH_DISPLAY] = \
                     self.latexmathdisplay_end_string
 
                 if self._include_xheaders:
                     self.xheaders.update({
-                        "x-mdpo-latexmath-start": self.latexmath_start_string,
-                        "x-mdpo-latexmath-end": self.latexmath_end_string,
-                        "x-mdpo-latexmathdisplay-start":
+                        'x-mdpo-latexmath-start': self.latexmath_start_string,
+                        'x-mdpo-latexmath-end': self.latexmath_end_string,
+                        'x-mdpo-latexmathdisplay-start':
                             self.latexmathdisplay_start_string,
-                        "x-mdpo-latexmathdisplay-end":
+                        'x-mdpo-latexmathdisplay-end':
                             self.latexmathdisplay_end_string,
                     })
 
-            if "wikilinks" in self.extensions:
+            if 'wikilinks' in self.extensions:
                 self.wikilink_start_string = kwargs.get(
-                    'wikilink_start_string', '[[')
+                    'wikilink_start_string', '[[',
+                )
                 self.wikilink_end_string = kwargs.get(
-                    'wikilink_end_string', ']]')
+                    'wikilink_end_string', ']]',
+                )
 
                 self._enterspan_replacer[md4c.SpanType.WIKILINK] = \
                     self.wikilink_start_string
@@ -169,26 +188,28 @@ class Md2Po:
 
                 if self._include_xheaders:
                     self.xheaders.update({
-                        "x-mdpo-wikilink-start": self.wikilink_start_string,
-                        "x-mdpo-wikilink-end": self.wikilink_end_string,
+                        'x-mdpo-wikilink-start': self.wikilink_start_string,
+                        'x-mdpo-wikilink-end': self.wikilink_end_string,
                     })
 
-            if "underline" in self.extensions:
+            if 'underline' in self.extensions:
                 # underline text is standarized with double '_'
                 self.underline_start_string = kwargs.get(
-                    'underline_start_string', '__')
+                    'underline_start_string', '__',
+                )
                 self._enterspan_replacer[md4c.SpanType.U] = \
                     self.underline_start_string
 
                 self.underline_end_string = kwargs.get(
-                    'underline_end_string', '__')
+                    'underline_end_string', '__',
+                )
                 self._leavespan_replacer[md4c.SpanType.U] = \
                     self.underline_end_string
 
                 if self._include_xheaders:
                     self.xheaders.update({
-                        "x-mdpo-underline-start": self.underline_start_string,
-                        "x-mdpo-underline-end": self.underline_end_string,
+                        'x-mdpo-underline-start': self.underline_start_string,
+                        'x-mdpo-underline-end': self.underline_end_string,
                     })
 
             # optimization to skip checking for
@@ -213,20 +234,37 @@ class Md2Po:
         self._uls_deep = 0
 
     def _save_msgid(self, msgid, tcomment=None, msgctxt=None):
-        if polib.POEntry(msgid=msgid) not in self.pofile:
-            entry = polib.POEntry(msgid=msgid, msgstr=self.msgstr,
-                                  comment=tcomment,
-                                  msgctxt=msgctxt)
+        entry = polib.POEntry(
+            msgid=msgid, msgstr=self.msgstr,
+            comment=tcomment,
+            msgctxt=msgctxt,
+        )
+        _equal_entry = find_entry_in_entries(
+            entry, self.pofile,
+            compare_obsolete=False,
+            compare_msgstr=False,
+        )
+        if _equal_entry and _equal_entry.msgstr:
+            entry.msgstr = _equal_entry.msgstr
+            if _equal_entry.fuzzy and not entry.fuzzy:
+                entry.flags.append('fuzzy')
+        if entry not in self.pofile:
             self.pofile.append(entry)
-        self.msgids.append(msgid)
+        self.found_entries.append(entry)
 
     def _save_current_msgid(self):
-        if self._current_msgid and ((not self._disable_next_line and
-                                     not self._disable) or
-                                    self._enable_next_line):
-            self._save_msgid(self._current_msgid,
-                             tcomment=self._current_tcomment,
-                             msgctxt=self._current_msgctxt)
+        if self._current_msgid and (
+            (
+                not self._disable_next_line and
+                not self._disable
+            ) or
+            self._enable_next_line
+        ):
+            self._save_msgid(
+                self._current_msgid,
+                tcomment=self._current_tcomment,
+                msgctxt=self._current_msgctxt,
+            )
         self._disable_next_line = False
         self._enable_next_line = False
         self._current_msgid = ''
@@ -234,7 +272,7 @@ class Md2Po:
         self._current_msgctxt = None
 
     def _process_command(self, text):
-        command, comment = search_html_command(text)
+        command, comment = parse_mdpo_html_command(text)
         if command is None:
             return
 
@@ -248,23 +286,29 @@ class Md2Po:
             self._enable_next_line = True
         elif command == 'translator':
             if not comment:
-                raise ValueError('You need to specify a string for the'
-                                 ' extracted comment with the command'
-                                 ' \'mdpo-translator\'.')
-            self._current_tcomment = comment.strip(" ")
+                raise ValueError(
+                    'You need to specify a string for the'
+                    ' extracted comment with the command'
+                    ' \'mdpo-translator\'.',
+                )
+            self._current_tcomment = comment.strip(' ')
         elif command == 'context':
             if not comment:
-                raise ValueError('You need to specify a string for the'
-                                 ' context with the command \'mdpo-context\'.')
-            self._current_msgctxt = comment.strip(" ")
+                raise ValueError(
+                    'You need to specify a string for the'
+                    ' context with the command \'mdpo-context\'.',
+                )
+            self._current_msgctxt = comment.strip(' ')
         elif command == 'include-codeblock':
             self._include_next_codeblock = True
         elif command == 'include':
             if not comment:
-                raise ValueError('You need to specify a message for the'
-                                 ' comment to include with the command'
-                                 ' \'mdpo-include\'.')
-            self._current_msgid = comment.strip(" ")
+                raise ValueError(
+                    'You need to specify a message for the'
+                    ' comment to include with the command'
+                    ' \'mdpo-include\'.',
+                )
+            self._current_msgid = comment.strip(' ')
             self._save_current_msgid()
 
     def enter_block(self, block, details):
@@ -312,7 +356,7 @@ class Md2Po:
                     pass
 
             if span.value == md4c.SpanType.A:
-                self._current_aspan_href = details["href"][0][1]
+                self._current_aspan_href = details['href'][0][1]
             elif span.value == md4c.SpanType.CODE:
                 self._inside_codespan = True
 
@@ -350,10 +394,11 @@ class Md2Po:
 
             if span.value == md4c.SpanType.A:
                 if self._current_aspan_href:
-                    self._current_msgid += "(%s%s)" % (
+                    self._current_msgid += '({}{})'.format(
                         self._current_aspan_href,
-                        "" if not details['title'] else ' "%s"' % (
-                            details['title'][0][1])
+                        '' if not details['title'] else ' "%s"' % (
+                            details['title'][0][1]
+                        ),
                     )
                     self._current_aspan_href = None
             elif span.value == md4c.SpanType.CODE:
@@ -361,15 +406,18 @@ class Md2Po:
                 self._codespan_start_index = None
                 # add backticks at the end for escape internal backticks
                 self._current_msgid += (
-                    self._codespan_backticks * self.code_end_string)
+                    self._codespan_backticks * self.code_end_string
+                )
                 self._codespan_backticks = None
             elif span.value == md4c.SpanType.IMG:
-                self._current_msgid += '![%s](%s' % (
-                    self._current_imgspan['text'], self._current_imgspan['src']
+                self._current_msgid += '![{}]({}'.format(
+                    self._current_imgspan['text'],
+                    self._current_imgspan['src'],
                 )
                 if self._current_imgspan['title']:
                     self._current_msgid += ' "%s"' % (
-                        self._current_imgspan['title'])
+                        self._current_imgspan['title']
+                    )
                 self._current_msgid += ')'
                 self._current_imgspan = {}
             elif span.value == md4c.SpanType.U:
@@ -379,7 +427,7 @@ class Md2Po:
         # print("TEXT:", text)
         if not self._inside_htmlblock:
             if not self._inside_codeblock:
-                if self._inside_liblock and text == "\n":
+                if self._inside_liblock and text == '\n':
                     text = ' '
                 if not self.plaintext:
                     if self._current_imgspan:
@@ -390,11 +438,13 @@ class Md2Po:
                         # internal backticks
                         self._codespan_backticks = min_not_max_chars_in_a_row(
                             self.code_start_string,
-                            text) - 1
-                        self._current_msgid = '%s%s%s' % (
+                            text,
+                        ) - 1
+                        self._current_msgid = '{}{}{}'.format(
                             self._current_msgid[:self._codespan_start_index],
                             self._codespan_backticks * self.code_start_string,
-                            self._current_msgid[self._codespan_start_index:])
+                            self._current_msgid[self._codespan_start_index:],
+                        )
                     elif text == self.italic_start_string:
                         text = self.italic_start_string_escaped
                     elif text == self.code_start_string:
@@ -404,13 +454,14 @@ class Md2Po:
                     elif text == self.code_end_string:   # pragma: no cover
                         text = self.code_end_string_escaped
                 if self._inside_pblock:
-                    text = text.replace("\n", " ")
+                    text = text.replace('\n', ' ')
                 if self._current_wikilink_target:
                     if text != self._current_wikilink_target:
                         # not self-referenced wikilink
-                        self._current_wikilink_target = '%s|%s' % (
+                        self._current_wikilink_target = '{}|{}'.format(
                             self._current_wikilink_target,
-                            text)
+                            text,
+                        )
                     return
                 self._current_msgid += text
             else:
@@ -419,8 +470,10 @@ class Md2Po:
         else:
             self._process_command(text)
 
-    def extract(self, po_filepath=None, save=False, mo_filepath=None,
-                encoding=None):
+    def extract(
+        self, po_filepath=None, save=False, mo_filepath=None, po_encoding=None,
+        md_encoding='utf-8',
+    ):
         _po_filepath = None
         if not po_filepath:
             po_filepath = ''
@@ -429,37 +482,63 @@ class Md2Po:
             if not os.path.exists(po_filepath):
                 po_filepath = ''
 
-        pofile_kwargs = (dict(autodetect_encoding=False, encoding=encoding) if
-                         encoding else {})
-        self.pofile = polib.pofile(po_filepath, wrapwidth=self.wrapwidth,
-                                   **pofile_kwargs)
+        pofile_kwargs = (
+            dict(autodetect_encoding=False, encoding=po_encoding)
+            if po_encoding else {}
+        )
+        self.pofile = polib.pofile(
+            po_filepath, wrapwidth=self.wrapwidth,
+            **pofile_kwargs,
+        )
 
-        parser = md4c.GenericParser(0,
-                                    **{ext: True for ext in self.extensions})
+        parser = md4c.GenericParser(
+            0,
+            **{ext: True for ext in self.extensions},
+        )
 
         def _parse(content):
-            parser.parse(content,
-                         self.enter_block,
-                         self.leave_block,
-                         self.enter_span,
-                         self.leave_span,
-                         self.text)
+            parser.parse(
+                content,
+                self.enter_block,
+                self.leave_block,
+                self.enter_span,
+                self.leave_span,
+                self.text,
+            )
 
         if hasattr(self, 'content'):
             _parse(self.content)
         else:
             for filepath in self.filepaths:
-                with open(filepath, "r") as f:
+                with open(filepath, encoding=md_encoding) as f:
                     content = f.read()
                 _parse(content)
                 self._disable_next_line = False
                 self._disable = False
                 self._enable_next_line = False
 
-        if self.mark_not_found_as_absolete:
+        if self.mark_not_found_as_obsolete:
             for entry in self.pofile:
-                if entry.msgid not in self.msgids:
-                    entry.obsolete = True
+                if entry not in self.found_entries:
+                    _equal_not_obsolete_found = find_entry_in_entries(
+                        entry, self.found_entries, compare_obsolete=False,
+                    )
+                    if _equal_not_obsolete_found:
+                        self.pofile.remove(entry)
+                    else:
+                        entry.obsolete = True
+                else:
+                    entry.obsolete = False
+        else:
+            for entry in self.pofile:
+                if entry not in self.found_entries:
+                    _equal_not_obsolete_found = find_entry_in_entries(
+                        entry, self.found_entries, compare_obsolete=False,
+                    )
+                    if _equal_not_obsolete_found:
+                        self.pofile.remove(entry)
+                    else:
+                        entry.obsolete = False
 
         if self._include_xheaders:
             self.pofile.metadata.update(self.xheaders)
@@ -471,16 +550,15 @@ class Md2Po:
         return self.pofile
 
 
-def markdown_to_pofile(glob_or_content, ignore=[], msgstr='',
-                       po_filepath=None, save=False, mo_filepath=None,
-                       plaintext=False, wrapwidth=78,
-                       mark_not_found_as_absolete=True,
-                       extensions=DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
-                       encoding=None, xheaders=False,
-                       include_codeblocks=False, **kwargs):
-    """
-    Extracts all the msgids from a string of Markdown content or a group
-    of files.
+def markdown_to_pofile(
+    glob_or_content, ignore=[], msgstr='', po_filepath=None, save=False,
+    mo_filepath=None, plaintext=False, wrapwidth=78,
+    mark_not_found_as_obsolete=True,
+    extensions=DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS, po_encoding=None,
+    md_encoding='utf-8', xheaders=False, include_codeblocks=False, **kwargs,
+):
+    """Extracts all the msgids from a string of Markdown content or a group of
+    files.
 
     Args:
         glob_or_content (str): Glob path to Markdown files or a string
@@ -494,7 +572,7 @@ def markdown_to_pofile(glob_or_content, ignore=[], msgstr='',
             instance where to dump the new msgids and that will be used
             as source checking not found strings that will be marked as
             obsolete if is the case (see ``save`` and
-            ``mark_not_found_as_absolete`` optional parameters).
+            ``mark_not_found_as_obsolete`` optional parameters).
         save (bool): Save the new content to the pofile indicated in the
             parameter ``po_filepath``.
         mo_filepath (str): The resulting pofile will be compiled to a mofile
@@ -511,14 +589,15 @@ def markdown_to_pofile(glob_or_content, ignore=[], msgstr='',
         wrapwidth (int): Wrap width for po file indicated at ``po_filepath``
             parameter. Only useful when the ``-w`` option was passed
             to xgettext.
-        mark_not_found_as_absolete (bool): The strings extracted from markdown
+        mark_not_found_as_obsolete (bool): The strings extracted from markdown
             that will not be found inside the provided pofile will be marked
             as obsolete.
         extensions (list): md4c extensions used to parse markdown content,
             formatted as a list of 'pymd4c' keyword arguments. You can see all
             available at `pymd4c repository <https://github.com/dominickpastore
             /pymd4c#parser-option-flags>`_.
-        encoding (bool): Resulting pofile encoding (autodetected by default).
+        po_encoding (str): Resulting pofile encoding.
+        md_encoding (str): Markdown content encoding.
         xheaders (bool): Indicates if the resulting pofile will have mdpo
             x-headers included. These only can be included if the parameter
             ``plaintext`` is ``False``.
@@ -529,10 +608,10 @@ def markdown_to_pofile(glob_or_content, ignore=[], msgstr='',
 
     Examples:
         >>> content = 'Some text with `inline code`'
-        >>> entries = markdown_to_pofile(content)
+        >>> entries = markdown_to_pofile(content, plaintext=True)
         >>> {e.msgid: e.msgstr for e in entries}
         {'Some text with inline code': ''}
-        >>> entries = markdown_to_pofile(content, plaintext=False)
+        >>> entries = markdown_to_pofile(content)
         >>> {e.msgid: e.msgstr for e in entries}
         {'Some text with `inline code`': ''}
         >>> entries = markdown_to_pofile(content, msgstr='Default message')
@@ -540,16 +619,15 @@ def markdown_to_pofile(glob_or_content, ignore=[], msgstr='',
         {'Some text with inline code': 'Default message'}
 
     Returns:
-        :class:`polib.POFile`:: Pofile instance with new msgids included.
-
+        :class:`polib.POFile` Pofile instance with new msgids included.
     """
     return Md2Po(
-        glob_or_content, ignore=ignore, msgstr=msgstr,
-        plaintext=plaintext, wrapwidth=wrapwidth,
-        mark_not_found_as_absolete=mark_not_found_as_absolete,
+        glob_or_content, ignore=ignore, msgstr=msgstr, plaintext=plaintext,
+        wrapwidth=wrapwidth,
+        mark_not_found_as_obsolete=mark_not_found_as_obsolete,
         extensions=extensions, xheaders=xheaders,
-        include_codeblocks=include_codeblocks, **kwargs
+        include_codeblocks=include_codeblocks, **kwargs,
     ).extract(
-        po_filepath=po_filepath, save=save,
-        mo_filepath=mo_filepath, encoding=encoding
+        po_filepath=po_filepath, save=save, mo_filepath=mo_filepath,
+        po_encoding=po_encoding, md_encoding=md_encoding,
     )
